/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   This class builds the GUI, instantiates
   all the program objects, and makes sure
   that every object has a reference to any
   other object that it must communicate with.

   This class instantiates the JFrame that
   is the source of the key events.
*/
public class CircleProgramWithMouseButtonsSlider
{
   /**
      This constructor instantiates the GUI.
   */
   public CircleProgramWithMouseButtonsSlider()
   {
      final JFrame jf = new JFrame("Circle Program With Mouse, Buttons, Slider");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      final CircleState cs = new CircleState();
      final CirclePanel cp = new CirclePanel(cs);
      final JButton jb1 = new JButton("Reset1");
      final JButton jb2 = new JButton("Reset2");
      final JSlider js  = new JSlider(5, 100, 20);
      jf.getContentPane().setLayout(new FlowLayout());
      jf.getContentPane().add(cp);
      jf.getContentPane().add(jb1);
      jf.getContentPane().add(jb2);
      jf.getContentPane().add(js);
      jf.pack();
      jf.setLocationRelativeTo(null);
      jf.requestFocusInWindow();
      jf.setVisible(true);

      cp.addKeyListener(new CircleKeyListener(cs, cp));
      cp.setFocusable(true); // without this, the panel never gets foscus

      //jf.addMouseListener(new CircleMouseListener(cs));
      //jf.addMouseMotionListener(new CircleMouseMotionListener(cs, cp));
      cp.addMouseListener(new CircleMouseListener(cs));
      cp.addMouseMotionListener(new CircleMouseMotionListener(cs, cp));

      jb1.addActionListener(new ButtonActionListener1(cs, cp));
      jb2.addActionListener(new ButtonActionListener2(cs, cp));

      js.addChangeListener(new SliderChangeListener(cs, cp));
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgramWithMouseButtonsSlider()
      );
   }//main()
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

    JFrame object
+----------------------+
|                      |    List<KeyListener>     CircleKeyListener object
|  List<KeyListener>---|------->+------+        +-------------------------+
|                      |        |      |        | CircleState cs          |
|                      |        +------+        | CirclePanel cp          |
|  addKeyListener()    |        |  ----|------->|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
GUI object that receives        |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM.                                        |                         |
                                                +-------------------------+
                                                 An object that implements
                                                 the KeyListener interface.
    KeyEvent object
  +-----------------+
  |  information    |
  |  from the OS    |      CircleState object         CirclePanel object
  |  about the      |      +-------------------+     +---------------------------+
  |  keyboard event |      | int xNew          |     | CircleState cs            |
  +-----------------+      | int yNew          |     |                           |    List<MouseListener>
                           | List<Point> trail |     | List<MouseListener>-------|------->+------+
                           | boolean reset     |     | List<MouseMotionListener>-|--+     |  ----|---------------+
   MouseEvent object       | boolean inside    |     |                           |  |     +------+               |
  +-----------------+      | int diameter      |     | paintComponent()          |  |                            |
  |  information    |      +-------------------+     +---------------------------+  | List<MouseMotionListener>  |
  |  from the OS    |                                GUI object that needs          +---->+------+               |
  |  about the      |                                to be repainted after                |  ----|---+           |
  |  mouse event    |                                an event occurs.                     +------+   |           |
  +-----------------+                                                                                |           |
                                                                                                     |           |
                            CircleMouseListener object          CircleMouseMotionListener object     |           |
  ActionEvent object       +---------------------------+        +---------------------------+        |           |
 +-----------------+       | CircleState cs            |        | CircleState cs            |        |           |
 |  information    |       |                           |        | CirclePanel cp            |<-------+           |
 |  from the OS    |       | mouseExited(MouseEvent e) |<--+    |                           |                    |
 |  about the      |       | mouseRelease(MouseEvent e)|   |    | mouseDragged(MouseEvent e)|                    |
 |  action event   |       | mousePressed(MouseEvent e)|   |    +---------------------------+                    |
 +-----------------+       +---------------------------+   |    An object that implements the                    |
                            An object that implements      |    MouseMotionListener interface.                   |
                            the MouseListener interface.   |                                                     |
 ChangeEvent                                               +-----------------------------------------------------+
+-------------+
| information |                                        JButton object
| about the   |                                         |                                            JButton object
| change event|        ActionListener object            |         ActionListener object               |
|             |     +-------------------------------+   |     +-------------------------------+       |
+-------------+     | CircleState cs                |   |     | CircleState cs                |       |
                    | CirclePanel cp                |<--+     | CirclePanel cp                |<------+
                    |                               |         |                               |
                    | actionPerformed(ActionEvent e)|         | actionPerformed(ActionEvent e)|
                    +-------------------------------+         +-------------------------------+
                     An object that implements                  An object that implements
                     the ActionListener interface.              the AxcionListener interface.


           ChangeListener object                                            JSlider object
        +-----------------------------+      List<ChangeListener>      +----------------------+
        | CircleState cs              |         +--------+             |                      |
        | CirclePanel cp              |<--------|---     |<------------|-List<ChangeListener> |
        |                             |         +--------+             |                      |
        | stateChanged(ChangeEvent e) |                                |                      |
        +-----------------------------+                                |                      |
         An object that implements                                     +----------------------+
         the ChangeListener interface.
*/